# Requires
require 'thor'
require 'OS/Finder'
require 'OS/DocGenerator'

# Implement the CLI sub-commands for OS.
module PackerFiles
 module CLI
   class OS < Thor

      # List all supported Operating systems.
      desc "list", "list all supported operating systems"
      def list
	Finder.new.class_names.each do |cl|
	   name = cl.name.gsub('PackerFiles::','')
	   if cl.versions.empty?
	      versions = 'Any'
	   else
	      versions = cl.versions.to_s
	   end
	   puts "#{name} => #{versions}"
	end
      end

      # Generate the documentation needed for a given class name
      desc "init", "Create a Template file for the specified OS"
      option :name,  :required => true,  :desc => 'OS Name', :type => :string
      option :force, :desc    => 'Overwrite existing File, if any', :type => :boolean,
                     :default => false
      option :file, :desc => 'Output file name', :default => 'PackerFile',
                    :type => :string 
      def init
         klass = Finder.new.class_name(options[:name])
	 if (File.exist?(options[:file]) && !options[:force])
            $stderr.puts "Cannot overwrite #{options[:file]} w/o --force"
	    return 1
         end
         DocGenerator.new(klass, options[:file]).Generate
      end 


   end
 end
end
