
# Requires

module PackerFiles

   module Core

      # The Hypervisor class just abstracts out various settings that are 
      # applicable on Hypervisors. It is upto the providers to ensure that
      # the appropriate Packer variables are generated for these settings.
      class Hypervisor

	 # CPU count
	 attr_accessor :cpu_count

	 # CPU Speed
	 attr_accessor :cpu_speed

	 # Enable CPU Hot plug?
	 attr_accessor :cpu_hot_plug

	 # CPU Execution Cap
	 attr_accessor :cpu_execution_cap

	 # RAM Size
	 attr_accessor :ram_size

	 # Video ram size
	 attr_accessor :video_ram_size

	 # Is ACPI enabled?
	 attr_accessor :acpi_enabled

	 # Should PAE (Processor Address Extensions) enabled?
	 attr_accessor :pae_enabled

	 # Should H/w Virtualization for Guest be enabled
	 attr_accessor :hw_virt_enabled

	 # Should nested paging for guest be enabled
	 attr_accessor :hw_nested_pages

	 # Should Hypervisor use large pages for this guest
	 attr_accessor :use_large_pages

	 # Should the guest run in unrestricted mode
	 attr_accessor :unrestricted_guest_mode

	 # Should 3d acceleration be enabled for Video
	 attr_accessor :video_3d_acceleration

	 # Should EFI firmware be used instead of bios
	 attr_accessor :use_efi

	 # Guest memory balloon size
	 attr_accessor :guest_balloon_size

	 # Custom object which can be used for whatever purpose 
         # it suits the caller
	 attr_reader :custom

	 # Documentation for this class 
	 def self.doc_file
	    PackerFiles.DirPath('Core/example/Hypervisor.txt').first
	 end

	 # Constructor initializes some sane values for various parameters.
	 def initialize
	    @cpu_count               = 1
	    @cpu_speed               = '1GHz'
	    @cpu_hot_plug            = false
	    @cpu_execution_cap       = 100
	    @ram_size	             = '1GiB'
	    @video_ram_size          = '64MiB'
	    @acpi_enabled            = true
	    @pae_enabled             = true
	    @hw_virt_enabled         = true
	    @use_large_pages         = true
	    @unrestricted_guest_mode = false
	    @video_3d_acceleration   = false
	    @use_efi		     = false
	    @guest_balloon_size      = '0MiB'
	    @custom                  = nil
	    yield self if block_given?
	 end

	 # The Custom object is filled with whatever is the result of the block
	 def Custom(&block)
	    @custom = block.call(self)
         end

      end # class HyperVisor

   end # Core

end # PackerFiles
