
# Requires
require_relative 'Base'

module PackerFiles
   module Core

      # Define the Locale class that can handle locales specified in a Packerfile.
      # Just the barebones attributes that are required for this class are specified
      # here. The conversion of these attributes into a OS build specific file is 
      # done by derived classes in the OS specific directories.
      class Locale < Base

	# Specify attributes
	attr_accessor :default  
	attr_accessor :language  
	attr_accessor :country  
	attr_accessor :supported

	# Documentation for this class 
	def self.doc_file
	   PackerFiles.DirPath('Core/example/Locale.txt').first
	end

	# Constructor to just specify accessor varibales
	def initialize
	  @supported = []
	  yield self if block_given?
	end

	# Normalize the various values into something useful. Usually 
	# this means adding UTF-8 into the default string. if desired
	def normalize(utf = true, str='.UTF-8')

	  # Concatenate language and country if the detailed notation
	  # instead of the simpler one is preferred.
	  if (!@language.nil? && !@country.nil?)
	     @default = @language + '_' + @country
	  end

	  # If you need to add UTF prefixes, then add them to the 
	  # default locale.
	  if (utf && !@default.end_with?(str))
	    @default = @default + str
	  end

	  # If you need to add UTF prefixes for all the supported
	  # locales, do the same.
	  @supported.map! {|x|
	    if (utf && !x.end_with?(str)) 
	      x + str
	    else
	     x
	    end 
	  }
	end

     end # Locale

   end # Core

end # PackerFiles
