
# Requires
require_relative 'Base'
require_relative 'Exceptions'

module PackerFiles

   module Core
     # Define the Network class that can handle Network specified in a Packerfile.
     # Just the barebones attributes that are required for this class are specified
     # here. The conversion of these attributes into a OS build specific file is 
     # done by derived classes in the OS specific directories.
     class Network < Base

       # Specify attributes
       attr_accessor :interface  
       attr_accessor :ip  
       attr_accessor :mask  
       attr_accessor :gateway  
       attr_accessor :dns  
       attr_accessor :register  
       attr_accessor :type  

       # Constants 
       IPV4   = 1
       IPV6   = 2
       STATIC = 4

       # Documentation for this class 
       def self.doc_file
	  PackerFiles.DirPath('Core/example/Network.txt').first
       end

       # Constructor to just specify accessor varibales
       def initialize
	 @dns         = []
	 yield self if block_given?
       end

       # Helper function to check if a static IP address is configured.
       def is_static?
	 return ((@type & STATIC) > 0 )
       end

       # Normalize the various values into something useful. The 
       # default implementation checks only errors
       def normalize
	raise NilException.new(self, 'interface') if @interface.nil?
	raise NilException.new(self, 'type')      if @type.nil?
	raise 'Mixing IPV4 and IPV6'    if ((@type & IPV4) > 0) && ((@type & IPV6) > 0)
	if ( (@type & STATIC) > 0 )
	   raise NilException.new(self, 'ip')       if @ip.nil?
	   raise NilException.new(self, 'mask')     if @mask.nil?
	   raise NilException.new(self, 'gateway')  if @gateway.nil?
	   raise NilException.new(self, 'dns')      if @dns.nil?
	   raise EmptyArrayException.new(self, 'dns')    if @dns.empty?
	else
	  @register = true
	end
       end

     end

   end # Core

end # PackerFiles

