
# Requires
require_relative 'Base'
require_relative 'Exceptions'
require 'Utils/AutoZone'

module PackerFiles
   module Core

     # Define the TimeZone class that can handle TimeZone specified in a Packerfile.
     # Just the barebones attributes that are required for this class are specified
     # here. The conversion of these attributes into a OS build specific file is 
     # done by derived classes in the OS specific directories.
      class TimeZone < Base

	# Specify attributes
	attr_accessor :utc  
	attr_accessor :zone
	attr_accessor :auto

	# Documentation for this class 
	def self.doc_file
	   PackerFiles.DirPath('Core/example/TimeZone.txt').first
	end

	# Constructor to just specify accessor varibales
	def initialize
	  @utc      = true
	  yield self if block_given?
	end

	# Normalize the various values into something useful. The 
	# default implementation checks only errors
	def normalize
	  raise NilException.new(self, 'zone') if @zone.nil? && @auto.nil?
	  if (@zone.nil? && @auto)
	    @zone = Utils::AutoZone.new.time_zone
	  end
	end

      end # TimeZone

   end # Core

end  # PackerFiles
