
# Requires
require_relative 'Base'
require_relative 'Exceptions'

module PackerFiles

   module Core

      # Define the User class that can handle User specified in a Packerfile.
      # Just the barebones attributes that are required for this class are specified
      # here. The conversion of these attributes into a OS build specific file is 
      # done by derived classes in the OS specific directories.
      class User < Base

	# Specify attributes
	attr_accessor :login  
	attr_accessor :name  
	attr_accessor :password  
	attr_accessor :uid  
	attr_accessor :groups  

	# Documentation for this class 
	def self.doc_file
	   PackerFiles.DirPath('Core/example/User.txt').first
	end

	# Constructor to just specify accessor varibales
	def initialize
	  @groups      = []
	  yield self if block_given?
	end

	# Normalize the various values into something useful. The 
	# default implementation checks only errors
	def normalize
	 raise NilException.new(self, 'login')    if @login.nil?
	 raise NilException.new(self, 'name')     if @name.nil?
	 raise NilException.new(self, 'password') if @password.nil?
	end

      end

   end # Core

end # PackerFiles
