# Requires
require 'uri'
require 'open-uri'
require 'nokogiri'
require "Core/CDImage"
require "Utils/AutoZone"

# An implementation for CDImage which lists all available Debian CD Mirrors
module PackerFiles
   module Debian
      class CD < Core::CDImageImpl
 
         # Index URL which contains all the Debian CD Mirror list
	 @@index  = 'https://www.debian.org/CD/http-ftp/'

         # Override method for getting the ISO name
	 def get_iso_name_pattern
	    'debian-@release@-@arch@-CD-1.iso'
         end

	 # Override method for getting checksums
	 def get_check_sum_pattern
	    'MD5SUMS'
	 end

	 # Override method for checksum type
	 def get_check_sum_type
	    'md5'
	 end

         # Override method for getting index URLs.
         def index_urls
	    master_index_urls + mirror_index_urls
         end

         # The Index URLs from the main debian CD Image server.
         private
         def master_index_urls
	 %w[
           http://cdimage.debian.org/debian-cd/@release@/@arch@/iso-cd/
           http://cdimage.debian.org/mirror/cdimage/archive/@release@/@arch@/iso-cd/
         ]
         end

	 # The Index URLs from other debian CD mirrors. The CD Mirrors only support CD
         # images and not DVD Images.
         private
         def mirror_index_urls
            
            # Use nokogiri to parse the index page, to get all the URLs that contain
            # country name [as detected by AutoZone]. Unfortunately this code can only
	    # be understood if HTML structure is visually inspected first. In short, 
            # debian mirrors for all the countries in the index file, looks like
            # <li> "Country" <a rel="nofollow" href="http://debian.xfree.com.ar/debian-cd/"> </li>
            page    = Nokogiri::HTML(open(@@index))
            country = Utils::AutoZone.new.JSON['country']
            nodes   = page.css('li').select {|node| node.text.include?(country)}
            arefs   = nodes.map {|node| node.css('a')}.flatten
	    mirrors = arefs.map {|aref| aref.attribute('href').value}
	    mirrors.map! {|m| URI::join(m, '@release@/@arch@/iso-cd/').to_s}
         end

      end
   end
end
