# Requires
require 'rubygems'

# OS class that handles details for generating Preseed files for Debian
# Servers.
module PackerFiles
 module Debian

   class Preseed 

     # Constructor. The preseed file name is the name of the file
     # that will be written to. 
     def initialize(filename)
        @preseed = filename
	create_file
     end 

     # Serializes the given object and write out the content
     # in the output file.
     def convert(obj, os_name)
        file     = erb_file(obj, os_name)
	content  = PackerFiles.evaluate_erb(file, { "obj" => obj })
	add_content(content)
     end

     # Evaluate a partially specified file name (should be in the Gem's
     # file list though) and add the content to the @preseed file
     def eval_file(file_name, hash)
       erb_file = PackerFiles.DirPath(file_name).first
       content  = PackerFiles.evaluate_erb(erb_file, hash)
       add_content(content)
     end 

     # Given an object, figure out the class name and return back the correct
     # erb file name.
     private
     def erb_file(obj, os_name)

       erb_file = File.join(os_name, obj.class.name.gsub(/.*::/,'') + '_erb.rb')
       erb_path = PackerFiles.DirPath(erb_file)
       
       if erb_path.nil? || erb_path.empty?
          raise RuntimeError.new("Template Path #{erb_file} does not exist")
       end
       return erb_path.first
     end

     # Create a new file with no content
     private
     def create_file
       basename = File.dirname(@preseed)
       FileUtils.mkdir_p(basename) if !Dir.exist?(basename)
       File.open(@preseed, 'w') {|f| f.write ''}
     end

     # Add content to the preseed file
     private
     def add_content(content)
       File.open(@preseed, 'a') {|f| f.write(content) }
     end

   end
 end
end 
