# Requires

# The Document generator class generates the template file for a 
# given OS class. 
module PackerFiles
   class DocGenerator

      # Constructor accepts a class name and the file name to write
      # the documentation to.
      def initialize(class_name, filename)
         @class_name = class_name
         @file_name  = filename
      end

      # Generate the documentation.This is done by looking at the 
      # type accessors that are defined by the OS class and then 
      # calling the doc_file function to figure out the ERB template.
      def Generate

	  # Generate content for all the type accessor classes.
	  content = []
          ancestry_class_list.each do |class_name|
	     hash  = class_name.type_accessors
	     next  if hash.nil?
             hash.each_pair { |func, type_data|
	        file = type_data.first.doc_file
	        hash = {'name' => func, 'mod' => PackerFiles}
	        doc  = PackerFiles.evaluate_erb(file, hash)
	        content += doc.split("\n").map {|line| '   ' + line  }
             }
          end

	  # Generate content for the top level structure.
	  top_doc_file = PackerFiles.DirPath('OS/example/Doc.txt').first
          os_name      = @class_name.name.gsub('PackerFiles::', '')
          hash         = Hash['os_name', os_name, 'content', content]
	  value        = PackerFiles.evaluate_erb(top_doc_file, hash)
	  File.write(@file_name, value)
      end

      # Get the complete hierarchy of classes until PackerFiles::Builder. 
      # This is to ensure that all type_accessors are captured at every
      # inheritance level.
      private 
      def ancestry_class_list
         index = @class_name.ancestors.index(PackerFiles::Builder)
         @class_name.ancestors.slice(0, index + 1)
      end

   end
end
