# Requires
require 'OS/CentOS/Server'
require_relative 'FedoraCD'

# OS class that handles details for generating Packer files for Fedora
# Servers.
module PackerFiles
 module RHEL

   class Fedora < PackerFiles::CentOS::Server

     # Constructor. Yield self for further initialization
     def initialize
	yield self if block_given?
     end 

     # Hook function called before normalizing CD Image
     def cd_image_hook
	self.CDImage.impl = PackerFiles::Fedora::CD.new
     end 

     # Hook for writing into preseed file.
     def kickstart_write_hook(objects)

	# Preseed file by evaluating templates @ Debian folder.
	kickstart = PackerFiles::CentOS::Kickstart.new(kickstart_file)

	# Evaluate Head 
	kickstart.eval_file('CentOS/Templates/Base_erb.rb', {})

        # Convert objects into Preseed configuration.
        objects.each do |obj|
	   kickstart.convert obj, 'CentOS/Templates'
        end

	# Evaluate tail
	kickstart.eval_file('RHEL/Templates/Fedora_Tail_erb.rb', {'obj' => self})

     end

     # Name of the preseed file
     def kickstart_file
        rel  = self.CDImage.release
        arch = self.CDImage.arch
        name = "kickstart-fedora-server-#{rel}-#{arch}.cfg"
	File.join(http_dir, name)
     end

     # Boot Command hook for Fedora
     def boot_command
        file  = PackerFiles.DirPath('RHEL/Templates/boot_command_fedora_erb.rb')
        hash  = { 'obj' => self, 'network' => self.Network }
        value = PackerFiles.evaluate_erb(file.first, hash)
        value.split("\n")
     end

  end
 end
end 
