# Requires
require "Core/CDImage"
require "Utils/AutoZone"
require 'nokogiri'
require 'open-uri'

# An implementation for CDImage For Fedora
module PackerFiles
   module Fedora
      class CD < Core::CDImageImpl

	 # Master URL
	 @@index = 'https://mirrors.fedoraproject.org/publiclist/Fedora/'
 
         # Override method for getting the ISO name
	 def get_iso_name_pattern
	    'Fedora-@release@-@arch@-DVD.iso'
         end

	 # Override method for getting checksums
	 def get_check_sum_pattern
	    'Fedora-@release@-@arch@-CHECKSUM'
	 end

	 # Override method for checksum type
	 def get_check_sum_type
	    'sha256'
	 end

         # Override method for getting index URLs. This is a dummy URL
	 # which is not accessible.
         def index_urls
	    default_urls + mirror_index_urls
         end

	 # Master URL list for fedora.
	 private
	 def default_urls
	 [
	 'https://archives.fedoraproject.org/pub/archive/fedora/linux/releases/@release@/Fedora/@arch@/iso/',
	 'https://archives.fedoraproject.org/pub/fedora/linux/releases/@release@/Fedora/@arch@/iso/'
	 ]
	 end
	
	 # List of Mirror URLs for Fedora. We use nokogiri to parse values from the Index URL
	 # and try to find a matching mirror for a given country.Since we are doing HTML Parsing
	 # the code below is found to fail sometime.
	 private
	 def mirror_index_urls
            page    = Nokogiri::HTML(open(@@index))
            country = Utils::AutoZone.new.JSON['country_code']
	    set     = page.css('td').select {|td| td.text.end_with?(country)}
	    urls    = set.map {|node|
	       rows  = node.parent.css('td[4]/table/tr')
	       row   = rows.select{|row| row.css('td[1]').text.match('Fedora Linux|Archive')}
	       if !row.empty?
	          links = row.map {|e| e.css('a').map {|link| link['href']}}.flatten.compact
		  links.select {|link| link.start_with?('http')}
	       end
            }.flatten
	    urls.map {|url|
	       url + '/releases/@release@/Fedora/@arch@/iso/'
	    }

	 end
      end
   end
end
