# Requires
require 'open-uri'
require 'uri'
require 'Utils/AutoZone'
require 'Utils/FastestURL'
require 'nokogiri'

# Auto configures the Ubuntu mirror list. This is useful in pre-seeding
# the fastest possible mirror.
module PackerFiles
 module Ubuntu
   class Mirrors

      # default ubuntu mirror
      @@default = 'http://archive.ubuntu.com/ubuntu'

      # Default ubuntu mirror list for any country.
      @@mirrors = 'http://mirrors.ubuntu.com/@country@.txt'

      # Default Master archive mirror list
      @@master  = 'https://launchpad.net/ubuntu/+archivemirrors'

      # accessors
      attr_reader :hosts
      attr_reader :best
      attr_reader :proxy

      # Constructor
      def initialize

         # A Single web page get more than this (in seconds) is SLOW  
         limit   = 1 

         # No. of trials for a web page
         trials  = 4  
   
	 # Get the name of the http proxy
	 @proxy   = ENV['http_proxy'] 

         # Get the host list of mirrors for this country
         code     = Utils::AutoZone.new.country_code.upcase
         @hosts   = country_mirror(code)
	 if @hosts.empty?
	    @hosts   = [@@default] if @hosts.empty?
	    @best    = URI::parse(@@default)
	 else
	    @hosts   = filter_status_unknown.compact
	    f        = Utils::FastestURL.new(limit, trials)
	    all      = f.best_urls(@hosts).flatten
	    begin
	       @best    = URI::parse(all.first)
	    rescue Exception => e
	       puts "Exception: #{e}, All: #{all.to_s}"
	       raise e
	    end
	 end
      end

      # Given the country code, get a list of mirror sites that match
      # the country code.
      private
      def country_mirror(code)
        url      = @@mirrors.gsub('@country@', code)
        contents = open(url) {|f| f.read}
        return contents.split("\n")
      end

      # Given a list of hosts which can act as mirrors, filter out those
      # hosts whose status is unknown, as they can cause updates and 
      # new installation packages to fail.
      private
      def filter_status_unknown
	
	# Parse the HTML from the master index archive.
	page = Nokogiri::HTML(open(@@master))

	# Iterate through the list of hosts
	@hosts.map {|host|
	   node = page.css('td').select {|node|
              aref = node.css('a')
              if aref.empty?
                false
              else
                aref.first['href'] == host
              end
           }
	   if node.empty?
	      nil
	   else
              text = node.first.next_element.next_element.text
	      if text.include?('Last update unknown')
		nil
	      else
	        host
	      end
	   end
	}
      end

   end #class Mirror

 end
end 
