# Requires
require 'OS/Debian/Server'
require_relative 'CD'
require_relative 'Mirrors'

# OS class that handles details for generating Packer files for Ubuntu
# Servers.
module PackerFiles
 module Ubuntu

   class Server < Debian::Server

     # Constructor. Yield self for further initialization
     def initialize
        super
     end 

     # Hook function called before normalizing CD Image
     def cd_image_hook
	self.CDImage.impl = Ubuntu::CD.new
     end 

     # Hook for writing into preseed file.
     def preseed_write_hook(objects)

	# Preseed file by evaluating templates @ Debian folder.
	preseed = Debian::Preseed.new(preseed_file)

        # Convert objects into Preseed configuration.
        objects.each do |obj|
	   if obj.is_a? Debian::Apt
	      preseed.convert obj, 'Ubuntu/Templates'
	   else
	      preseed.convert obj, 'Debian/Templates'
	   end
        end

	# Generate fastest debian mirrors
	preseed.eval_file 'Debian/Templates/Mirrors_erb.rb', 
			  {'obj' => Ubuntu::Mirrors.new}

        # Return the preseed object for further customization 
        # by derived classes.
	return preseed

     end

     # Name of the preseed file
     def preseed_file
        rel  = self.CDImage.release
        arch = self.CDImage.arch
        name = "preseed-ubuntu-server-#{rel}-#{arch}.cfg"
	File.join(http_dir, name)
     end

     # Boot Command Override
     def boot_command
        file  = PackerFiles.DirPath('Ubuntu/Templates/boot_command_erb.rb')
	hash  = { 'obj' => self, 'network' => self.Network }
        value = PackerFiles.evaluate_erb(file.first, hash)
	value.split("\n")
     end

     # Guest OS Type override
     def guest_os_type
	return "Ubuntu_64" if (self.CDImage.arch == "amd64")
	return "Ubuntu"
     end

     # VMWare Guest OS Type override
     def vmware_guest_os_type
	return "ubuntu-64" if (self.CDImage.arch == "amd64")
	return "ubuntu"
     end

  end
 end
end 
