# Requires
require 'json'
require_relative 'Base'
require_relative 'ChefSoloHelper'

# The Chef Solo class is used for executing a series of Cookbooks
# inside the VM, using chef solo provisioner.
module PackerFiles
   module Provision 
      class ChefSolo < Base 

         # Various accessors
	 hash_variable  :cookbook_paths, Array, :convert_cookbook_paths
	 hash_variable  :data_bags_path
	 hash_variable  :encrypted_data_bag_secret_path
         hash_variable  :execute_command
	 hash_variable  :install_command
	 hash_variable  :json, String, :convert_to_json
	 hash_variable  :prevent_sudo, String, :convert_sudo_to_boolean
	 hash_variable  :remote_cookbook_paths
	 hash_variable  :roles_path
	 hash_variable  :run_list, Array
	 hash_variable  :skip_install, String, :convert_skip_to_boolean
	 hash_variable  :staging_directory
	 
	 # Attribute accessor which is not converted into a hash serializer
	 #attr_accessor  :download_cookbooks 

	 # JSON converter
	 def convert_to_json
	    # Check if it is a string and if so parse it into JSON
	    begin
	       return JSON.parse(self.json)
	    rescue JSON::ParserError => e
	       return JSON.parse(File.read(self.json))
	    end
	 end

         # Cookbook Path converter
	 def convert_cookbook_paths
	    self.cookbook_paths.map {|p| File.absolute_path(p)}
	 end
  
	 # Boolean converter for prevent_sudo
	 def convert_sudo_to_boolean
	    (self.prevent_sudo.to_i > 0) ? true : false
	 end
 
	 # Boolean converter for skip_install
	 def convert_skip_to_boolean
	    (self.skip_install.to_i > 0) ? true : false
	 end

         # Constructor yields self
         def initialize(&block)
	    yield self if block_given?
	    @type = 'chef-solo'
         end 

         # If chef-solo has to work, all the dependent cookbooks also
         # needs to be found in one of the cookbook folders. So we need
         # to intercept the to_hash call to extract cookbooks inside a 
	 # .tar.gz bundle.
         def to_hash(*args)
	   
	    # The Provisioner usually passes the OS class as it's first
	    # argument
	    os   = args[0]

	    # Some cookbook paths are special in the sense, they are .tar.gz
	    # bundles. The code fragment handles that special case.
	    convert_cookbook_paths.each_with_index do |path, index|
               next if !path.end_with?('.tar.gz')
	       temp   = Dir.mktmpdir('dep', os.OutDir) 
	       helper = ChefSoloHelper.new(path)
	       local  = helper.download_files(os.OutDir)
	       helper.extract_files(temp, local)
	       @cookbook_paths.push(temp)
	    end

	    # Remove all the cookbooks that end with .tar.gz
	    @cookbook_paths.delete_if {|cb| cb.end_with?('.tar.gz')}

	    # call back the original to_hash function.
	    super
         end

     end
  end
end
