# Requires
require 'uri'
require 'open-uri'
require 'rubygems/package'
require 'zlib'
require 'fileutils'

# Chef Solo helper is useful to download [remote tar.gz] files into a local
# folder.
module PackerFiles
  module Provision
    class ChefSoloHelper

       # Just remember the name of the file 
       attr_accessor :tar_gz_file

       # Initialize the helper with the path of the .tar.gz file
       def initialize(tar_gz_file)
          @tar_gz_file = tar_gz_file
       end

      # Given a top level directory, and the tar_gz_file, download
      # it locally if it is a remote file.
      def download_files(top_dir)
	 url = URI::parse(@tar_gz_file)
	 if url.scheme.nil?
	    return @tar_gz_file
         else
	    name    = File.join(top_dir, File.basename(@tar_gz_file))
	    content = open(@tar_gz_file) {|f| f.read}
	    File.write(name, content)
	    return name
	 end
      end 

      # Given a list of .tar.gz files, untar and ungzip them
      def extract_files(dest_dir, file)
         untar(ungzip(file), dest_dir)
      end

      # un-gzips the given IO, returning the decompressed version 
      # as a StringIO
      private
      def ungzip(tarfile)
         zip   = Zlib::GzipReader.open(tarfile)
         unzip = StringIO.new(zip.read)
         zip.close
         unzip
      end

      # untars the given IO into the specified directory
      private
      def untar(io, dest_dir)
         Gem::Package::TarReader.new io do |tar|
            tar.each do |tarfile|
               dest = File.join(dest_dir, tarfile.full_name)
               if tarfile.directory?
                  FileUtils.mkdir_p(dest)
	          next
               end
               dir = File.dirname(dest)
               FileUtils.mkdir_p dir unless File.directory?(dir)
               File.open dest, "wb" do |f|
                  f.print tarfile.read
               end
            end
         end
      end

    end
  end
end
