# Requires
require_relative 'Copy'
require_relative 'InlineScript'
require_relative 'ExternalScript'
require_relative 'ChefSolo'

# Top level class for managing Provisioners in a OS Spec file
module PackerFiles
   class Provisioners 

      # Documentation for this class 
      def self.doc_file
         PackerFiles.DirPath('Provision/example/Provisioners.txt').first
      end

      # List of registered provisioners
      attr_accessor :list

      # Constructor just yields self for further initialization
      def initialize(&block)
	 @list = []
         yield self if block_given?   
      end

      # Enable File Copy Provisioner to be called multiple times
      # with order preservation.
      def Copy(&block)
	 @list.push(Provision::Copy.new(&block))
      end

      # Enable Inline Script Provisioner to be called multiple times
      # with order preservation.
      def InlineScript(&block)
	 @list.push(Provision::InlineScript.new(&block))
      end

      # Enable External Script Provisioner to be called multiple times
      # with order preservation.
      def ExternalScript(&block)
	 @list.push(Provision::ExternalScript.new(&block))
      end

      # Enable Chef Solo Provisioner to be called multiple times
      # with order preservation.
      def ChefSolo(&block)
	 @list.push(Provision::ChefSolo.new(&block))
      end

      # Convert self into Packer JSON format.
      def convert(gen, os)
	 @list.unshift(EnableRootLoginScript(os))
	 @list.push(DisableRootLoginScript(os))
         @list.compact.each do |prov|
	    prov.os = os
	    hash    = prov.to_hash(os)
	    gen.provisioners.push(hash) if !hash.nil?
         end
      end

      # Create a inline script command to enable root logins
      private
      def EnableRootLoginScript(os)
	cmd    = os.run_command_as_root
        enable = os.enable_root_for_user
	if !cmd.nil? && !enable.nil?
	   return Provision::InlineScript.new do |script|
	      script.execute_command = cmd
	      script.inline          = enable
	   end
	end
      end

      # Create a inline script command to disable root logins
      private
      def DisableRootLoginScript(os)
	cmd     = os.run_command_as_root
        disable = os.disable_root_for_user
	if !cmd.nil? && !disable.nil?
	   return Provision::InlineScript.new do |script|
	      script.execute_command = cmd
	      script.inline          = disable
	   end
	end
      end
 
   end
end
