# Requires
require 'rubygems'
require 'open-uri'
require 'json'

module PackerFiles

  module Utils

     # AutoZone handles the automatic detection of a time zone via
     # web based GEO Lookups.
     class AutoZone

	 # URL for maxmind timezone specifier
	 @@url = 'https://raw.githubusercontent.com/maxmind/geoip-api-c/master/timezone/timezone.txt'

	 # List of URLs for Geo-IP lookup
	 @@geo = [
	    'http://www.telize.com/geoip',
	    'http://freegeoip.net/json'
	 ]

	 # JSON object cache to prevent multiple internet lookups.
	 @@cache = nil

	 # Return back the Cached JSON
	 def self.Cache
	    @@cache
	 end

	 # Return the JSON from the web services, to do what ever.
	 def JSON
	  
	   # Return the cache if it is populated already
	   return @@cache if !@@cache.nil?

	   # Cache Miss! Let us get it from Internet.
	   @@geo.each do |url|
	      begin
		contents = open(url) {|f| f.read }
		@@cache  = JSON.parse(contents)
		break
	      rescue
		next
	      end
	   end
	   return @@cache
	 end

	 # Sugar coated country code
	 def country_code
	    return self.JSON['country_code']
	 end

	 # Sugar coated country_area code
	 def country_area_code
	    json  = self.JSON
	    return json['country_code'], json['area_code']
	 end

	 # Sugar coated perfect time zone look-up
	 def time_zone
       
	    # Get the RAW JSON
	    json  = self.JSON

	    # If it already contains a timezone, good enough and move on.
	    return json['timezone'] if json.has_key?('timezone')

	    # If not need more lookups with max-mind DB. Cache it for further
	    # use. 
	    country = json['country_code']
	    area    = json['area_code']
	    open(@@url) {|f|
	     f.each_line do |line|
	       arr = line.split("\t")
	       # Return the Time Zone if there is an exact match of country and area
	       if (arr[0] == country && arr[1] == area)
		  @@cache['timezone'] = arr[2]
		  break
	       # Return the Time Zone if there is an exact match of country and if 
	       # area cannot be determined reliably.
	       elsif (arr[0] == country && (arr[1].empty? || area.empty?))
		  @@cache['timezone'] = arr[2]
		  break
	       end
	     end 
	   }
	   return @@cache['timezone']
	 end

     end # AutoZone

  end # Utils

end # PackerFiles
