# Requires
require_relative "../Core/Exceptions"

module PackerFiles

   module Utils

      # Exception class used to indicate that a required type accessor is
      # missing.
      class MissingAccessor < Core::Error
	 def initialize(accessor_name, class_name)
	    self.message = "Missing #{accessor_name} Section for #{class_name}"
	 end
      end

      # The Type accessor class is used as a meta-class for any class.
      class TypeAccessor
      
       # The Meta-class Idiom. All functions within this block are meta-class 
       # functions that cannot be invoked on the object, but can be invoked 
       # on the class.
       class << self

	 # Custom accessor to create accessors from type name. It is also
	 # Possible to set the name of the accessor. By default it is the 
	 # name of the type.
	 def type_accessor(type, name     = type.name.gsub(/.*::/, ''), 
				 optional = false)

	    # Save the type and name in a array hash, which will be used later.
	    @types ||= Hash.new
	    @types[name] = [type, optional]

	    # Doing a class_eval creates a new method of "name"
	    class_eval do
     
	      # Read/Write accessor which accepts blocks for initialization.
	      define_method("#{name}") do |arg = nil, &block|
		if (self.class.proc_from(&block).nil?)
		   instance_variable_get("@#{name}")
		else
		   obj = type.new(&block)
		   instance_variable_set("@#{name}", obj)
		end
	      end # define_method
	    end # class_eval

	 end

	 # Return the Hash that is used to track registered type accessors
	 def type_accessors
	   @types
	 end
	 
	 # Given a block, convert it into a proc. if no block is given
	 # then it is a nil object that is returned
	 def proc_from(&block)
	   if block_given?
	      Proc.new
	   else
	      nil
	   end
	 end

       end # class << self

      end # TypeAccessor

   end # Utils

end # PackerFiles
