# Requires
require 'Utils/TypeAccessor'
require 'Core/Hypervisor'

module PackerFiles

   module Virtual

      # Internal List of Hypervisors that are supported. All of them Just override
      # the Custom Object call.
      class I_VirtualBox < Core::Hypervisor
         def Custom(&block)
	    @custom = VirtualBox.new(&block)
         end
      end

      class I_VMWare < Core::Hypervisor
         def Custom(&block)
	    @custom = VMWare.new(&block)
         end
      end

      class I_KVM < Core::Hypervisor
         def Custom(&block)
	    @custom = KVM.new(&block)
         end
      end

      # Top level class for managing hypervisors in a OS Spec file
      class Hypervisors  < Utils::TypeAccessor

	 # Documentation for this class 
	 def self.doc_file
	    PackerFiles.DirPath('Virtual/example/Hypervisors.txt').first
	 end

	 # Specify accessors for various supported Hypervisors
	 type_accessor I_VirtualBox, :VirtualBox
	 type_accessor I_VMWare,     :VMWare
	 type_accessor I_KVM,        :KVM

	 # Constructor just yields self for further initialization
	 def initialize(&block)
	    yield self if block_given?   
	 end

	 # Given a generator and a OS class derived from builder, convert
	 # it into hashes suitable for packer json file generation
	 def convert(gen, builder)

	   # Convert parameters into hash for registered Hypervisors
	   if !self.VirtualBox.nil?
	      hash = builder.vbox_converter_hook(gen)
	      gen.builders.push(hash) if !hash.nil?
	   end
	   if !self.VMWare.nil?
	      hash = builder.vmware_converter_hook(gen)
	      gen.builders.push(hash) if !hash.nil?
	   end
	   if !self.KVM.nil?
	      hash = builder.kvm_converter_hook(gen)
	      gen.builders.push(hash) if !hash.nil?
	   end

	 end

      end # Hypervisors

   end  # Virtual

end # PackerFiles

